# -*- coding: utf-8 -*-
# file: __init__.py
# time: 14:08 06/04/2024
# author: YANG, HENG <hy345@exeter.ac.uk> (杨恒)
# github: https://github.com/yangheng95
# huggingface: https://huggingface.co/yangheng
# google scholar: https://scholar.google.com/citations?user=NPq5a_0AAAAJ&hl=en
# Copyright (C) 2019-2025. All Rights Reserved.
"""
This package contains model definitions for various tasks.
"""

from .classification.model import *
from .mlm.model import *
from .regression.model import *
from .seq2seq.model import *
from .rna_design.model import *
from .embedding.model import *
from .augmentation.model import *
from .baselines import (
    OmniCNNBaseline,
    OmniRNNBaseline,
    OmniBPNetBaseline,
    OmniBasenjiBaseline,
    OmniDeepSTARRBaseline,
    OmniGenericBaseline,
    create_baseline,
)
